package gregapi.oredict;

import static gregapi.data.CS.F;
import static gregapi.data.CS.T;
import static gregapi.data.CS.U;
import gregapi.code.ICondition;
import gregapi.code.TagData;

/**
 * @author Gregorius Techneticies
 * 
 * A Collection of Classes, which check for certain Material Conditions.
 */
public class OreDictMaterialCondition {
	public static ICondition<OreDictMaterial> tag		(TagData... aTags)		{return new TagDataContainsAll(aTags);}
	public static ICondition<OreDictMaterial> tagnor	(TagData... aTags)		{return new TagDataContainsNone(aTags);}
	public static ICondition<OreDictMaterial> fullpulver()						{return FullPulver.INSTANCE;}
	public static ICondition<OreDictMaterial> fullforge()						{return FullForge.INSTANCE;}
	public static ICondition<OreDictMaterial> prefix	(OreDictPrefix aPrefix)	{return new TagDataPrefixGenerates(aPrefix);}
	public static ICondition<OreDictMaterial> meltmin	(long aMeltingPoint)	{return new MeltingPointMin(aMeltingPoint);}
	public static ICondition<OreDictMaterial> meltmax	(long aMeltingPoint)	{return new MeltingPointMax(aMeltingPoint);}
	public static ICondition<OreDictMaterial> boilmin	(long aBoilingPoint)	{return new BoilingPointMin(aBoilingPoint);}
	public static ICondition<OreDictMaterial> boilmax	(long aBoilingPoint)	{return new BoilingPointMax(aBoilingPoint);}
	public static ICondition<OreDictMaterial> plasmin	(long aPlasmaPoint)		{return new PlasmaPointMin(aPlasmaPoint);}
	public static ICondition<OreDictMaterial> plasmax	(long aPlasmaPoint)		{return new PlasmaPointMax(aPlasmaPoint);}
	public static ICondition<OreDictMaterial> qualmin	(long aMinQuality)		{return new QualityMin(aMinQuality);}
	public static ICondition<OreDictMaterial> qualmax	(long aMaxQuality)		{return new QualityMax(aMaxQuality);}
	
	private static class FullPulver implements ICondition<OreDictMaterial> {
		public static final FullPulver INSTANCE = new FullPulver();
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return aMaterial.mTargetPulver.mAmount >= U;}
	}
	
	private static class FullForge implements ICondition<OreDictMaterial> {
		public static final FullForge INSTANCE = new FullForge();
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return aMaterial.mTargetForging.mAmount >= U;}
	}
	
	private static class TagDataContainsAll implements ICondition<OreDictMaterial> {
		private final TagData[] mTags;
		public TagDataContainsAll(TagData... aTags) {mTags = aTags;}
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return aMaterial.containsAll(mTags);}
	}
	
	private static class TagDataContainsNone implements ICondition<OreDictMaterial> {
		private final TagData[] mTags;
		public TagDataContainsNone(TagData... aTags) {mTags = aTags;}
		@Override public boolean isTrue(OreDictMaterial aMaterial) {for (TagData tTag : mTags) if (aMaterial.contains(tTag)) return F; return T;}
	}
	
	private static class TagDataPrefixGenerates implements ICondition<OreDictMaterial> {
		private final OreDictPrefix mPrefix;
		public TagDataPrefixGenerates(OreDictPrefix aPrefix) {mPrefix = aPrefix;}
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return mPrefix.canGenerateItem(aMaterial);}
	}
	
	private static class QualityMin implements ICondition<OreDictMaterial> {
		private final long mMinQuality;
		public QualityMin(long aMinQuality) {mMinQuality = aMinQuality;}
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return aMaterial.mToolQuality >= mMinQuality;}
	}
	
	private static class QualityMax implements ICondition<OreDictMaterial> {
		private final long mMaxQuality;
		public QualityMax(long aMaxQuality) {mMaxQuality = aMaxQuality;}
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return aMaterial.mToolQuality >= mMaxQuality;}
	}
	
	private static class MeltingPointMin implements ICondition<OreDictMaterial> {
		private final long mMeltingPoint;
		public MeltingPointMin(long aMeltingPoint) {mMeltingPoint = aMeltingPoint;}
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return aMaterial.mMeltingPoint >= mMeltingPoint;}
	}
	
	private static class MeltingPointMax implements ICondition<OreDictMaterial> {
		private final long mMeltingPoint;
		public MeltingPointMax(long aMeltingPoint) {mMeltingPoint = aMeltingPoint;}
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return aMaterial.mMeltingPoint <= mMeltingPoint;}
	}
	
	private static class BoilingPointMin implements ICondition<OreDictMaterial> {
		private final long mBoilingPoint;
		public BoilingPointMin(long aBoilingPoint) {mBoilingPoint = aBoilingPoint;}
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return aMaterial.mBoilingPoint >= mBoilingPoint;}
	}
	
	private static class BoilingPointMax implements ICondition<OreDictMaterial> {
		private final long mBoilingPoint;
		public BoilingPointMax(long aBoilingPoint) {mBoilingPoint = aBoilingPoint;}
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return aMaterial.mBoilingPoint <= mBoilingPoint;}
	}
	
	private static class PlasmaPointMin implements ICondition<OreDictMaterial> {
		private final long mPlasmaPoint;
		public PlasmaPointMin(long aPlasmaPoint) {mPlasmaPoint = aPlasmaPoint;}
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return aMaterial.mPlasmaPoint >= mPlasmaPoint;}
	}
	
	private static class PlasmaPointMax implements ICondition<OreDictMaterial> {
		private final long mPlasmaPoint;
		public PlasmaPointMax(long aPlasmaPoint) {mPlasmaPoint = aPlasmaPoint;}
		@Override public boolean isTrue(OreDictMaterial aMaterial) {return aMaterial.mPlasmaPoint <= mPlasmaPoint;}
	}
}